@@ -32,7 +32,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "libedataserver/e-flag.h"
 
 #include <glib/gi18n-lib.h>
 
@@ -59,7 +58,8 @@ typedef struct {
 } CamelSqlite3File;
 
 static gint
-call_old_file_Sync (CamelSqlite3File *cFile, gint flags)
+call_old_file_Sync (CamelSqlite3File *cFile,
+                    gint flags)
 {
 	g_return_val_if_fail (old_vfs != NULL, SQLITE_ERROR);
 	g_return_val_if_fail (cFile != NULL, SQLITE_ERROR);
@@ -68,36 +68,48 @@ call_old_file_Sync (CamelSqlite3File *cFile, gint flags)
 	return cFile->old_vfs_file->pMethods->xSync (cFile->old_vfs_file, flags);
 }
 
+typedef struct {
+	GCond *cond;
+	GMutex *mutex;
+	gboolean is_set;
+} SyncDone;
+
 struct SyncRequestData
 {
 	CamelSqlite3File *cFile;
 	guint32 flags;
-	EFlag *sync_op; /* not NULL when waiting for a finish; will be freed by the caller */
+	SyncDone *done; /* not NULL when waiting for a finish; will be freed by the caller */
 };
 
 static void
-sync_request_thread_cb (gpointer task_data, gpointer null_data)
+sync_request_thread_cb (gpointer task_data,
+                        gpointer null_data)
 {
 	struct SyncRequestData *sync_data = task_data;
-	EFlag *sync_op;
+	SyncDone *done;
 
 	g_return_if_fail (sync_data != NULL);
 	g_return_if_fail (sync_data->cFile != NULL);
 
 	call_old_file_Sync (sync_data->cFile, sync_data->flags);
 
-	sync_op = sync_data->sync_op;
+	done = sync_data->done;
 	g_free (sync_data);
 
-	if (sync_op)
-		e_flag_set (sync_op);
+	if (done != NULL) {
+		g_mutex_lock (done->mutex);
+		done->is_set = TRUE;
+		g_cond_broadcast (done->cond);
+		g_mutex_unlock (done->mutex);
+	}
 }
 
 static void
-sync_push_request (CamelSqlite3File *cFile, gboolean wait_for_finish)
+sync_push_request (CamelSqlite3File *cFile,
+                   gboolean wait_for_finish)
 {
 	struct SyncRequestData *data;
-	EFlag *sync_op = NULL;
+	SyncDone *done = NULL;
 	GError *error = NULL;
 
 	g_return_if_fail (cFile != NULL);
@@ -105,13 +117,17 @@ sync_push_request (CamelSqlite3File *cFile, gboolean wait_for_finish)
 
 	g_static_rec_mutex_lock (&cFile->sync_mutex);
 
-	if (wait_for_finish)
-		sync_op = e_flag_new ();
+	if (wait_for_finish) {
+		done = g_slice_new (SyncDone);
+		done->cond = g_cond_new ();
+		done->mutex = g_mutex_new ();
+		done->is_set = FALSE;
+	}
 
 	data = g_new0 (struct SyncRequestData, 1);
 	data->cFile = cFile;
 	data->flags = cFile->flags;
-	data->sync_op = sync_op;
+	data->done = done;
 
 	cFile->flags = 0;
 
@@ -123,15 +139,24 @@ sync_push_request (CamelSqlite3File *cFile, gboolean wait_for_finish)
 		g_warning ("%s: Failed to push to thread pool: %s\n", G_STRFUNC, error->message);
 		g_error_free (error);
 
-		if (sync_op)
-			e_flag_free (sync_op);
+		if (done != NULL) {
+			g_cond_free (done->cond);
+			g_mutex_free (done->mutex);
+			g_slice_free (SyncDone, done);
+		}
 
 		return;
 	}
 
-	if (sync_op) {
-		e_flag_wait (sync_op);
-		e_flag_free (sync_op);
+	if (done != NULL) {
+		g_mutex_lock (done->mutex);
+		while (!done->is_set)
+			g_cond_wait (done->cond, done->mutex);
+		g_mutex_unlock (done->mutex);
+
+		g_cond_free (done->cond);
+		g_mutex_free (done->mutex);
+		g_slice_free (SyncDone, done);
 	}
 }
 
@@ -177,7 +202,8 @@ def_subclassed (xDeviceCharacteristics, (sqlite3_file *pFile), (cFile->old_vfs_f
 #undef def_subclassed
 
 static gint
-camel_sqlite3_file_xCheckReservedLock (sqlite3_file *pFile, gint *pResOut)
+camel_sqlite3_file_xCheckReservedLock (sqlite3_file *pFile,
+                                       gint *pResOut)
 {
 	CamelSqlite3File *cFile;
 
@@ -232,7 +258,8 @@ camel_sqlite3_file_xClose (sqlite3_file *pFile)
 }
 
 static gint
-camel_sqlite3_file_xSync (sqlite3_file *pFile, gint flags)
+camel_sqlite3_file_xSync (sqlite3_file *pFile,
+                          gint flags)
 {
 	CamelSqlite3File *cFile;
 
@@ -261,7 +288,11 @@ camel_sqlite3_file_xSync (sqlite3_file *pFile, gint flags)
 }
 
 static gint
-camel_sqlite3_vfs_xOpen (sqlite3_vfs *pVfs, const gchar *zPath, sqlite3_file *pFile, gint flags, gint *pOutFlags)
+camel_sqlite3_vfs_xOpen (sqlite3_vfs *pVfs,
+                         const gchar *zPath,
+                         sqlite3_file *pFile,
+                         gint flags,
+                         gint *pOutFlags)
 {
 	static GStaticRecMutex only_once_lock = G_STATIC_REC_MUTEX_INIT;
 	static sqlite3_io_methods io_methods = {0};
@@ -288,7 +319,7 @@ camel_sqlite3_vfs_xOpen (sqlite3_vfs *pVfs, const gchar *zPath, sqlite3_file *pF
 		sync_pool = g_thread_pool_new (sync_request_thread_cb, NULL, 2, FALSE, NULL);
 
 	/* cFile->old_vfs_file->pMethods is NULL when open failed for some reason,
-	   thus do not initialize our structure when do not know the version */
+	 * thus do not initialize our structure when do not know the version */
 	if (io_methods.xClose == NULL && cFile->old_vfs_file->pMethods) {
 		/* initialize our subclass function only once */
 		io_methods.iVersion = cFile->old_vfs_file->pMethods->iVersion;
@@ -365,8 +396,8 @@ struct _CamelDBPrivate {
 static gint
 cdb_sql_exec (sqlite3 *db,
               const gchar *stmt,
-	      gint (*callback)(gpointer ,gint,gchar **,gchar **),
-	      gpointer data,
+              gint (*callback)(gpointer ,gint,gchar **,gchar **),
+              gpointer data,
               GError **error)
 {
 	gchar *errmsg = NULL;
@@ -402,10 +433,12 @@ cdb_sql_exec (sqlite3 *db,
 }
 
 /* checks whether string 'where' contains whole word 'what',
-   case insensitively (ascii, not utf8, same as 'LIKE' in SQLite3)
+ * case insensitively (ascii, not utf8, same as 'LIKE' in SQLite3)
 */
 static void
-cdb_match_func (sqlite3_context *ctx, gint nArgs, sqlite3_value **values)
+cdb_match_func (sqlite3_context *ctx,
+                gint nArgs,
+                sqlite3_value **values)
 {
 	gboolean matches = FALSE;
 	const gchar *what, *where;
@@ -548,7 +581,10 @@ camel_db_close (CamelDB *cdb)
  * Since: 2.24
  **/
 gint
-camel_db_set_collate (CamelDB *cdb, const gchar *col, const gchar *collate, CamelDBCollate func)
+camel_db_set_collate (CamelDB *cdb,
+                      const gchar *col,
+                      const gchar *collate,
+                      CamelDBCollate func)
 {
 		gint ret = 0;
 
@@ -678,7 +714,7 @@ camel_db_add_to_transaction (CamelDB *cdb,
  **/
 gint
 camel_db_transaction_command (CamelDB *cdb,
-                              GSList *qry_list,
+                              GList *qry_list,
                               GError **error)
 {
 	gint ret;
@@ -699,7 +735,7 @@ camel_db_transaction_command (CamelDB *cdb,
 		ret = cdb_sql_exec (cdb->db, query, NULL, NULL, error);
 		if (ret)
 			goto end;
-		qry_list = g_slist_next (qry_list);
+		qry_list = g_list_next (qry_list);
 	}
 
 	ret = cdb_sql_exec (cdb->db, "COMMIT", NULL, NULL, error);
@@ -710,11 +746,14 @@ end:
 }
 
 static gint
-count_cb (gpointer data, gint argc, gchar **argv, gchar **azColName)
+count_cb (gpointer data,
+          gint argc,
+          gchar **argv,
+          gchar **azColName)
 {
 	gint i;
 
-	for (i=0; i<argc; i++) {
+	for (i = 0; i < argc; i++) {
 		if (strstr(azColName[i], "COUNT")) {
 			*(guint32 *)data = argv [i] ? strtoul (argv [i], NULL, 10) : 0;
 		}
@@ -1053,30 +1092,42 @@ camel_db_delete_uid_from_vfolder_transaction (CamelDB *db,
 }
 
 static gint
-read_uids_callback (gpointer ref, gint ncol, gchar ** cols, gchar ** name)
+read_uids_callback (gpointer ref_array,
+                    gint ncol,
+                    gchar **cols,
+                    gchar **name)
 {
-	GPtrArray *array = (GPtrArray *) ref;
+	GPtrArray *array = ref_array;
 
-	#if 0
-	gint i;
-	for (i = 0; i < ncol; ++i) {
-		if (!name[i] || !cols[i])
-			continue;
+	g_return_val_if_fail (ncol == 1, 0);
 
-		if (!strcmp (name [i], "uid"))
-			g_ptr_array_add (array, (gchar *) (camel_pstring_strdup (cols[i])));
-	}
-	#else
-		if (cols[0])
-			g_ptr_array_add (array, (gchar *) (camel_pstring_strdup (cols[0])));
-	#endif
+	if (cols[0])
+		g_ptr_array_add (array, (gchar *) (camel_pstring_strdup (cols[0])));
 
-	 return 0;
+	return 0;
+}
+
+static gint
+read_uids_to_hash_callback (gpointer ref_hash,
+                            gint ncol,
+                            gchar **cols,
+                            gchar **name)
+{
+	GHashTable *hash = ref_hash;
+
+	g_return_val_if_fail (ncol == 2, 0);
+
+	if (cols[0])
+		g_hash_table_insert (hash, (gchar *) camel_pstring_strdup (cols[0]), GUINT_TO_POINTER (cols[1] ? strtoul (cols[1], NULL, 10) : 0));
+
+	return 0;
 }
 
 /**
  * camel_db_get_folder_uids:
  *
+ * Fills hash with uid->GUINT_TO_POINTER (flag)
+ *
  * Since: 2.24
  **/
 gint
@@ -1084,15 +1135,15 @@ camel_db_get_folder_uids (CamelDB *db,
                           const gchar *folder_name,
                           const gchar *sort_by,
                           const gchar *collate,
-                          GPtrArray *array,
+                          GHashTable *hash,
                           GError **error)
 {
 	 gchar *sel_query;
 	 gint ret;
 
-	 sel_query = sqlite3_mprintf("SELECT uid FROM %Q%s%s%s%s", folder_name, sort_by ? " order by " : "", sort_by ? sort_by: "", (sort_by && collate) ? " collate " : "", (sort_by && collate) ? collate : "");
+	 sel_query = sqlite3_mprintf("SELECT uid,flags FROM %Q%s%s%s%s", folder_name, sort_by ? " order by " : "", sort_by ? sort_by: "", (sort_by && collate) ? " collate " : "", (sort_by && collate) ? collate : "");
 
-	 ret = camel_db_select (db, sel_query, read_uids_callback, array, error);
+	 ret = camel_db_select (db, sel_query, read_uids_to_hash_callback, hash, error);
 	 sqlite3_free (sel_query);
 
 	 return ret;
@@ -1152,25 +1203,41 @@ camel_db_get_folder_deleted_uids (CamelDB *db,
 	 return array;
 }
 
-static gint
-read_preview_callback (gpointer ref, gint ncol, gchar ** cols, gchar ** name)
+struct ReadPreviewData
 {
-	GHashTable *hash = (GHashTable *) ref;
-	const gchar *uid=NULL;
-	gchar *msg=NULL;
+	GHashTable *columns_hash;
+	GHashTable *hash;
+};
+
+static gint
+read_preview_callback (gpointer ref,
+                       gint ncol,
+                       gchar **cols,
+                       gchar **name)
+{
+	struct ReadPreviewData *rpd = ref;
+	const gchar *uid = NULL;
+	gchar *msg = NULL;
 	gint i;
 
 	for (i = 0; i < ncol; ++i) {
 		if (!name[i] || !cols[i])
 			continue;
 
-		if (!strcmp (name [i], "uid"))
-			uid = camel_pstring_strdup (cols[i]);
-		else if (!strcmp (name [i], "preview"))
-			msg = g_strdup (cols[i]);
+		switch (camel_db_get_column_ident (&rpd->columns_hash, i, ncol, name)) {
+			case CAMEL_DB_COLUMN_UID:
+				uid = camel_pstring_strdup (cols[i]);
+				break;
+			case CAMEL_DB_COLUMN_PREVIEW:
+				msg = g_strdup (cols[i]);
+				break;
+			default:
+				g_warn_if_reached ();
+				break;
+		}
 	}
 
-	g_hash_table_insert (hash, (gchar *) uid, msg);
+	g_hash_table_insert (rpd->hash, (gchar *) uid, msg);
 
 	return 0;
 }
@@ -1185,21 +1252,28 @@ camel_db_get_folder_preview (CamelDB *db,
                              const gchar *folder_name,
                              GError **error)
 {
-	 gchar *sel_query;
-	 gint ret;
-	 GHashTable *hash = g_hash_table_new (g_str_hash, g_str_equal);
+	gchar *sel_query;
+	gint ret;
+	struct ReadPreviewData rpd;
+	GHashTable *hash = g_hash_table_new (g_str_hash, g_str_equal);
 
-	 sel_query = sqlite3_mprintf("SELECT uid, preview FROM '%q_preview'", folder_name);
+	sel_query = sqlite3_mprintf ("SELECT uid, preview FROM '%q_preview'", folder_name);
 
-	 ret = camel_db_select (db, sel_query, read_preview_callback, hash, error);
-	 sqlite3_free (sel_query);
+	rpd.columns_hash = NULL;
+	rpd.hash = hash;
 
-	 if (!g_hash_table_size (hash) || ret != 0) {
-		 g_hash_table_destroy (hash);
-		 hash = NULL;
-	 }
+	ret = camel_db_select (db, sel_query, read_preview_callback, &rpd, error);
+	sqlite3_free (sel_query);
 
-	 return hash;
+	if (rpd.columns_hash)
+		g_hash_table_destroy (rpd.columns_hash);
+
+	if (!g_hash_table_size (hash) || ret != 0) {
+		g_hash_table_destroy (hash);
+		hash = NULL;
+	}
+
+	return hash;
 }
 
 /**
@@ -1226,24 +1300,15 @@ camel_db_write_preview_record (CamelDB *db,
 }
 
 static gint
-read_vuids_callback (gpointer ref, gint ncol, gchar ** cols, gchar ** name)
+read_vuids_callback (gpointer ref,
+                     gint ncol,
+                     gchar **cols,
+                     gchar **name)
 {
 	GPtrArray *array = (GPtrArray *) ref;
 
-	#if 0
-	gint i;
-
-	for (i = 0; i < ncol; ++i) {
-		if (!name[i] || !cols[i] || strlen (cols[i]) <= 8)
-			continue;
-
-		if (!strcmp (name [i], "vuid"))
-			g_ptr_array_add (array, (gchar *) (camel_pstring_strdup (cols[i]+8)));
-	}
-	#else
-		if (cols[0] && strlen (cols[0]) > 8)
-			g_ptr_array_add (array, (gchar *) (camel_pstring_strdup (cols[0]+8)));
-	#endif
+	if (cols[0] && strlen (cols[0]) > 8)
+		g_ptr_array_add (array, (gchar *) (camel_pstring_strdup (cols[0]+8)));
 
 	return 0;
 }
@@ -1432,7 +1497,7 @@ camel_db_migrate_folder_prepare (CamelDB *cdb,
 		ret = camel_db_add_to_transaction (cdb, table_creation_query, error);
 		sqlite3_free (table_creation_query);
 
-		table_creation_query = sqlite3_mprintf ("INSERT INTO 'mem.%q' SELECT uid , flags , msg_type , read , deleted , replied , important , junk , attachment , msg_security , size , dsent , dreceived , subject , mail_from , mail_to , mail_cc , mlist , followup_flag , followup_completed_on , followup_due_by , part , labels , usertags , cinfo , bdata , strftime(\"%%s\", 'now'), strftime(\"%%s\", 'now') FROM %Q", folder_name, folder_name);
+		table_creation_query = sqlite3_mprintf ("INSERT INTO 'mem.%q' SELECT uid , flags , msg_type , read , deleted , replied , important , junk , attachment , dirty , size , dsent , dreceived , subject , mail_from , mail_to , mail_cc , mlist , followup_flag , followup_completed_on , followup_due_by , part , labels , usertags , cinfo , bdata , strftime(\"%%s\", 'now'), strftime(\"%%s\", 'now') FROM %Q", folder_name, folder_name);
 		ret = camel_db_add_to_transaction (cdb, table_creation_query, error);
 		sqlite3_free (table_creation_query);
 
@@ -1461,13 +1526,27 @@ camel_db_migrate_folder_recreate (CamelDB *cdb,
 	/* Migration stage two: writing back the old data */
 
 	if (version < 2) {
+		GError *local_error = NULL;
+
 		table_creation_query = sqlite3_mprintf ("INSERT INTO %Q SELECT uid , flags , msg_type , read , deleted , replied , important , junk , attachment , dirty , size , dsent , dreceived , subject , mail_from , mail_to , mail_cc , mlist , followup_flag , followup_completed_on , followup_due_by , part , labels , usertags , cinfo , bdata, created, modified FROM 'mem.%q'", folder_name, folder_name);
-		ret = camel_db_add_to_transaction (cdb, table_creation_query, error);
+		ret = camel_db_add_to_transaction (cdb, table_creation_query, &local_error);
 		sqlite3_free (table_creation_query);
 
-		table_creation_query = sqlite3_mprintf ("DROP TABLE 'mem.%q'", folder_name);
-		ret = camel_db_add_to_transaction (cdb, table_creation_query, error);
-		sqlite3_free (table_creation_query);
+		if (!local_error) {
+			table_creation_query = sqlite3_mprintf ("DROP TABLE 'mem.%q'", folder_name);
+			ret = camel_db_add_to_transaction (cdb, table_creation_query, &local_error);
+			sqlite3_free (table_creation_query);
+		}
+
+		if (local_error) {
+			if (local_error->message && strstr (local_error->message, "no such table") != NULL) {
+				/* ignore 'no such table' errors here */
+				g_clear_error (&local_error);
+				ret = 0;
+			} else {
+				g_propagate_error (error, local_error);
+			}
+		}
 	}
 
 	/* Add later version migrations here */
@@ -1534,7 +1613,10 @@ camel_db_write_folder_version (CamelDB *cdb,
 }
 
 static gint
-read_version_callback (gpointer ref, gint ncol, gchar ** cols, gchar ** name)
+read_version_callback (gpointer ref,
+                       gint ncol,
+                       gchar **cols,
+                       gchar **name)
 {
 	gint *version = (gint *) ref;
 
@@ -1739,10 +1821,19 @@ camel_db_write_folder_info_record (CamelDB *cdb,
 	return ret;
 }
 
-static gint
-read_fir_callback (gpointer  ref, gint ncol, gchar ** cols, gchar ** name)
+struct ReadFirData
 {
-	CamelFIRecord *record = *(CamelFIRecord **) ref;
+	GHashTable *columns_hash;
+	CamelFIRecord *record;
+};
+
+static gint
+read_fir_callback (gpointer ref,
+                   gint ncol,
+                   gchar **cols,
+                   gchar **name)
+{
+	struct ReadFirData *rfd = ref;
 	gint i;
 
 	d(g_print ("\nread_fir_callback called \n"));
@@ -1751,39 +1842,47 @@ read_fir_callback (gpointer  ref, gint ncol, gchar ** cols, gchar ** name)
 		if (!name[i] || !cols[i])
 			continue;
 
-		if (!strcmp (name [i], "folder_name"))
-			record->folder_name = g_strdup (cols[i]);
-
-		else if (!strcmp (name [i], "version"))
-			record->version = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-
-		else if (!strcmp (name [i], "flags"))
-			record->flags = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-
-		else if (!strcmp (name [i], "nextuid"))
-			record->nextuid = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-
-		else if (!strcmp (name [i], "time"))
-			record->time = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-
-		else if (!strcmp (name [i], "saved_count"))
-			record->saved_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-
-		else if (!strcmp (name [i], "unread_count"))
-			record->unread_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-
-		else if (!strcmp (name [i], "deleted_count"))
-			record->deleted_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-
-		else if (!strcmp (name [i], "junk_count"))
-			record->junk_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-		else if (!strcmp (name [i], "visible_count"))
-			record->visible_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-		else if (!strcmp (name [i], "jnd_count"))
-			record->jnd_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
-		else if (!strcmp (name [i], "bdata"))
-			record->bdata = g_strdup (cols[i]);
-
+		switch (camel_db_get_column_ident (&rfd->columns_hash, i, ncol, name)) {
+			case CAMEL_DB_COLUMN_FOLDER_NAME:
+				rfd->record->folder_name = g_strdup (cols[i]);
+				break;
+			case CAMEL_DB_COLUMN_VERSION:
+				rfd->record->version = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_FLAGS:
+				rfd->record->flags = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_NEXTUID:
+				rfd->record->nextuid = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_TIME:
+				rfd->record->time = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_SAVED_COUNT:
+				rfd->record->saved_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_UNREAD_COUNT:
+				rfd->record->unread_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_DELETED_COUNT:
+				rfd->record->deleted_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_JUNK_COUNT:
+				rfd->record->junk_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_VISIBLE_COUNT:
+				rfd->record->visible_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_JND_COUNT:
+				rfd->record->jnd_count = cols[i] ? strtoul (cols[i], NULL, 10) : 0;
+				break;
+			case CAMEL_DB_COLUMN_BDATA:
+				rfd->record->bdata = g_strdup (cols[i]);
+				break;
+			default:
+				g_warn_if_reached ();
+				break;
+		}
 	}
 
 	return 0;
@@ -1797,17 +1896,24 @@ read_fir_callback (gpointer  ref, gint ncol, gchar ** cols, gchar ** name)
 gint
 camel_db_read_folder_info_record (CamelDB *cdb,
                                   const gchar *folder_name,
-                                  CamelFIRecord **record,
+                                  CamelFIRecord *record,
                                   GError **error)
 {
+	struct ReadFirData rfd;
 	gchar *query;
 	gint ret;
 
-	query = sqlite3_mprintf ("SELECT * FROM folders WHERE folder_name = %Q", folder_name);
-	ret = camel_db_select (cdb, query, read_fir_callback, record, error);
+	rfd.columns_hash = NULL;
+	rfd.record = record;
 
+	query = sqlite3_mprintf ("SELECT * FROM folders WHERE folder_name = %Q", folder_name);
+	ret = camel_db_select (cdb, query, read_fir_callback, &rfd, error);
 	sqlite3_free (query);
-	return (ret);
+
+	if (rfd.columns_hash)
+		g_hash_table_destroy (rfd.columns_hash);
+
+	return ret;
 }
 
 /**
@@ -1926,7 +2032,7 @@ camel_db_delete_uid (CamelDB *cdb,
 static gint
 cdb_delete_ids (CamelDB *cdb,
                 const gchar *folder_name,
-                GSList *uids,
+                GList *uids,
                 const gchar *uid_prefix,
                 const gchar *field,
                 GError **error)
@@ -1936,7 +2042,7 @@ cdb_delete_ids (CamelDB *cdb,
 	gchar *tab;
 	gboolean first = TRUE;
 	GString *str = g_string_new ("DELETE FROM ");
-	GSList *iterator;
+	GList *iterator;
 	GString *ins_str = NULL;
 
 	if (strcmp (field, "vuid") != 0)
@@ -2007,7 +2113,7 @@ cdb_delete_ids (CamelDB *cdb,
 gint
 camel_db_delete_uids (CamelDB *cdb,
                       const gchar *folder_name,
-                      GSList *uids,
+                      GList *uids,
                       GError **error)
 {
 	if (!uids || !uids->data)
@@ -2025,7 +2131,7 @@ gint
 camel_db_delete_vuids (CamelDB *cdb,
                        const gchar *folder_name,
                        const gchar *hash,
-                       GSList *uids,
+                       GList *uids,
                        GError **error)
 {
 	return cdb_delete_ids (cdb, folder_name, uids, hash, "vuid", error);
@@ -2268,26 +2374,6 @@ camel_db_get_column_name (const gchar *raw_name)
 
 }
 
-/**
- * camel_db_migrate_vfolders_to_14:
- *
- * Since: 2.24
- **/
-gint
-camel_db_migrate_vfolders_to_14 (CamelDB *cdb,
-                                 const gchar *folder,
-                                 GError **error)
-{
-	gchar *cmd = sqlite3_mprintf ("ALTER TABLE %Q ADD COLUMN flags INTEGER", folder);
-	gint ret;
-
-	ret = camel_db_command (cdb, cmd, error);
-	sqlite3_free (cmd);
-
-	CAMEL_DB_RELEASE_SQLITE_MEMORY;
-	return ret;
-}
-
 /**
  * camel_db_start_in_memory_transactions:
  *
@@ -2338,3 +2424,101 @@ camel_db_flush_in_memory_transactions (CamelDB *cdb,
 
 	return ret;
 }
+
+static struct _known_column_names {
+	const gchar *name;
+	CamelDBKnownColumnNames ident;
+} known_column_names[] = {
+	{ "attachment",			CAMEL_DB_COLUMN_ATTACHMENT },
+	{ "bdata",			CAMEL_DB_COLUMN_BDATA },
+	{ "bodystructure",		CAMEL_DB_COLUMN_BODYSTRUCTURE },
+	{ "cinfo",			CAMEL_DB_COLUMN_CINFO },
+	{ "deleted",			CAMEL_DB_COLUMN_DELETED },
+	{ "deleted_count",		CAMEL_DB_COLUMN_DELETED_COUNT },
+	{ "dreceived",			CAMEL_DB_COLUMN_DRECEIVED },
+	{ "dsent",			CAMEL_DB_COLUMN_DSENT },
+	{ "flags",			CAMEL_DB_COLUMN_FLAGS },
+	{ "folder_name",		CAMEL_DB_COLUMN_FOLDER_NAME },
+	{ "followup_completed_on",	CAMEL_DB_COLUMN_FOLLOWUP_COMPLETED_ON },
+	{ "followup_due_by",		CAMEL_DB_COLUMN_FOLLOWUP_DUE_BY },
+	{ "followup_flag",		CAMEL_DB_COLUMN_FOLLOWUP_FLAG },
+	{ "important",			CAMEL_DB_COLUMN_IMPORTANT },
+	{ "jnd_count",			CAMEL_DB_COLUMN_JND_COUNT },
+	{ "junk",			CAMEL_DB_COLUMN_JUNK },
+	{ "junk_count",			CAMEL_DB_COLUMN_JUNK_COUNT },
+	{ "labels",			CAMEL_DB_COLUMN_LABELS },
+	{ "mail_cc",			CAMEL_DB_COLUMN_MAIL_CC },
+	{ "mail_from",			CAMEL_DB_COLUMN_MAIL_FROM },
+	{ "mail_to",			CAMEL_DB_COLUMN_MAIL_TO },
+	{ "mlist",			CAMEL_DB_COLUMN_MLIST },
+	{ "nextuid",			CAMEL_DB_COLUMN_NEXTUID },
+	{ "part",			CAMEL_DB_COLUMN_PART },
+	{ "preview",			CAMEL_DB_COLUMN_PREVIEW },
+	{ "read",			CAMEL_DB_COLUMN_READ },
+	{ "replied",			CAMEL_DB_COLUMN_REPLIED },
+	{ "saved_count",		CAMEL_DB_COLUMN_SAVED_COUNT },
+	{ "size",			CAMEL_DB_COLUMN_SIZE },
+	{ "subject",			CAMEL_DB_COLUMN_SUBJECT },
+	{ "time",			CAMEL_DB_COLUMN_TIME },
+	{ "uid",			CAMEL_DB_COLUMN_UID },
+	{ "unread_count",		CAMEL_DB_COLUMN_UNREAD_COUNT },
+	{ "usertags",			CAMEL_DB_COLUMN_USERTAGS },
+	{ "version",			CAMEL_DB_COLUMN_VERSION },
+	{ "visible_count",		CAMEL_DB_COLUMN_VISIBLE_COUNT },
+	{ "vuid",			CAMEL_DB_COLUMN_VUID }
+};
+
+/**
+ * camel_db_get_column_ident:
+ *
+ * Traverses column name from index @index into an enum
+ * #CamelDBKnownColumnNames value.  The @col_names contains @ncols columns.
+ * First time this is called is created the @hash from col_names indexes into
+ * the enum, and this is reused for every other call.  The function expects
+ * that column names are returned always in the same order.  When all rows
+ * are read the @hash table can be freed with g_hash_table_destroy().
+ *
+ * Since: 3.4
+ **/
+CamelDBKnownColumnNames
+camel_db_get_column_ident (GHashTable **hash,
+                           gint index,
+                           gint ncols,
+                           gchar **col_names)
+{
+	gpointer value = NULL;
+
+	g_return_val_if_fail (hash != NULL, CAMEL_DB_COLUMN_UNKNOWN);
+	g_return_val_if_fail (col_names != NULL, CAMEL_DB_COLUMN_UNKNOWN);
+	g_return_val_if_fail (ncols > 0, CAMEL_DB_COLUMN_UNKNOWN);
+	g_return_val_if_fail (index >= 0, CAMEL_DB_COLUMN_UNKNOWN);
+	g_return_val_if_fail (index < ncols, CAMEL_DB_COLUMN_UNKNOWN);
+
+	if (!*hash) {
+		gint ii, jj, from, max = G_N_ELEMENTS (known_column_names);
+
+		*hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+		for (ii = 0, jj = 0; ii < ncols; ii++) {
+			const gchar *name = col_names[ii];
+			gboolean first = TRUE;
+
+			if (!name)
+				continue;
+
+			for (from = jj; first || jj != from; jj = (jj + 1) % max, first = FALSE) {
+				if (g_str_equal (name, known_column_names[jj].name)) {
+					g_hash_table_insert (*hash, GINT_TO_POINTER (ii), GINT_TO_POINTER (known_column_names[jj].ident));
+					break;
+				}
+			}
+
+			if (from == jj && !first)
+				g_warning ("%s: missing column name '%s' in a list of known columns", G_STRFUNC, name);
+		}
+	}
+
+	g_return_val_if_fail (g_hash_table_lookup_extended (*hash, GINT_TO_POINTER (index), NULL, &value), CAMEL_DB_COLUMN_UNKNOWN);
+
+	return GPOINTER_TO_INT (value);
+}
